clear all
set rmsg on

local content = 1
local clicks  = 2

mata
mata set matastrict on

class clicks {
    private:
        class AssociativeArray scalar agents
        class AssociativeArray scalar items
        class AssociativeArray scalar pr
        
        real scalar  Nagents
        real scalar  newscycle
        real scalar  pr_good
        real scalar  tdim
        real scalar  baseclicks
        real scalar  botclicks
        
        void         is_pr()
        void         is_posint()
        void         setup()
        void         populate_agents()
        void         populate_news()
        void         news_step()
        void         agent_step()
        void         pr()
        
    public:
        transmorphic Nagents()
        transmorphic newscycle()
        transmorphic pr_good()
        transmorphic tdim()
        transmorphic baseclicks()
        transmorphic botclicks()
        void         run()
        transmorphic mood()
        transmorphic items()
}

transmorphic clicks::items(| string rowvector names ) 
{
    real matrix res
    real rowvector idx
    real scalar i
    
    res = items.keys()
    res = res,J(rows(res), 1, .)
    for (i=1 ; i <= rows(res); i++) {
        res[i,4] = items.get(res[|i,1 \ i,3|])
    }
    _sort(res, (1,2,3))
    
    if (args() == 1) {
        if (cols(names) != 4) {
            _error(3300, "4 names must be specified")
        }
        idx = st_addvar("float", names)

        if (st_nobs() < rows(res) ) {
            st_addobs(rows(res) - st_nobs())
        }
        st_store(.,idx,res)
    }
    else {
        return(res)
    }
}

transmorphic clicks::mood(| string rowvector names)
{
    real scalar i, t, c
    real rowvector key, idx
    real matrix res
    
    res = J(tdim+1,10,0)
    for (t=0 ; t <= tdim; t++) {
        for (i = 1 ; i<= Nagents; i++) {
            key = i,t
            c = agents.get(key)
            res[t+1,c] = res[t+1,c] + 1
        }
    }
    if (args()==1) {
        if (cols(names) != 10) {
            _error(3300, "10 names must be specified")
        }
        idx = st_addvar("float", names)

        if (st_nobs() < rows(res) ) {
            st_addobs(rows(res) - st_nobs())
        }
        st_store(.,idx,res)
    }
    else {
        return(res)
    }
}

void clicks::agent_step(real scalar i, real scalar t)
{
    real scalar u, j, pref, prob
    real rowvector key
    
    key = i, t-1
    pref = agents.get(key)
    key = i, t
    agents.put(key,pref)
    
    u = runiform(1,1)
    prob = 0
    for(j=t+1; j <= t + newscycle; j++) {
        prob = prob + pr.get(j)
        if (u < prob) {
            key = j, t, `content'
            if (items.get(key)) {
                key = i,t
                agents.put(key,min((agents.get(key) +1, 10)))
            }
            else {
                key = i,t
                agents.put(key,max((agents.get(key)-1, 1)))
            }
            key = j, t, `clicks'
            items.put(key,items.get(key)+1)
            break
        }
    }
}

void clicks::news_step(real scalar t) 
{
    real scalar i, content, good
    real rowvector key
    
    for(i = t+1 ; i <= t + newscycle - 1; i++) {
        key = i, t-1, `content'
        content = items.get(key)
        key = i, t, `content'
        items.put(key, content)
        
        key = i, t-1, `clicks'
        content = items.get(key)
        key = i, t, `clicks'
        items.put(key,content)
    }
    
    key = t + newscycle, t, `content'
    good = runiform(1,1) < pr_good
    items.put(key, good)
    key = t + newscycle, t, `clicks'
    items.put(key, ceil(runiform(1,1)*baseclicks) + (1-good)*botclicks)
}

void clicks::pr(real scalar t)
{
    real scalar i, n, n_clicks
    real colvector key
    
    n_clicks = 0 
    
    for(i = t+1 ; i <= t + newscycle ; i++) {
        key = i,t,`clicks'
        n = items.get(key)
        n_clicks = n_clicks + n
    }
    

    for(i = t+1 ; i <= t + newscycle ; i++) {
        key = i,t,`clicks'
        n = items.get(key)
        pr.put(i, n/n_clicks)
    }
}

void clicks::is_posint(real scalar val, string scalar name, | string scalar zero_ok)
{
    string scalar errmsg
    if (args() == 2) {
        if (val <= 0 | mod(val,1)!= 0) {
            errmsg = name + " can only be a positive integer"
            _error(3300, errmsg)
        }
    }
    else {
    if (val < 0 | mod(val,1)!= 0) {
            errmsg = name + " can only be zero or a positive integer"
            _error(3300, errmsg)
        }    
    }
}

void clicks::is_pr(real scalar val, string scalar name)
{
    string scalar errmsg
    
    if (val > 1 | val < 0) {
        errmsg = name + " must remain between 0 and 1"
        _error(3300, errmsg)
    }
}

transmorphic clicks::Nagents(| real scalar dim) 
{
    if (args() == 1) {
        is_posint(dim, "dim")
        Nagents = dim
    }
    else {
        return(Nagents)
    }
}

transmorphic clicks::newscycle(| real scalar dim) 
{
    if (args() == 1) {
        is_posint(dim, "dim")
        newscycle = dim
    }
    else {
        return(newscycle)
    }
}

transmorphic clicks::pr_good(| real scalar dim) 
{
    if (args() == 1) {
        is_pr(dim, "dim")
        pr_good = dim
    }
    else {
        return(pr_good)
    }
}

transmorphic clicks::tdim(| real scalar dim) 
{
    if (args() == 1) {
        is_posint(dim, "dim")
        tdim = dim
    }
    else {
        return(tdim)
    }
}

transmorphic clicks::baseclicks(| real scalar dim) 
{
    if (args() == 1) {
        is_posint(dim, "dim")
        baseclicks = dim
    }
    else {
        return(baseclicks)
    }
}

transmorphic clicks::botclicks(| real scalar dim) 
{
    if (args() == 1) {
        is_posint(dim, "dim", "zero_ok")
        botclicks = dim
    }
    else {
        return(botclicks)
    }
}

void clicks::setup()
{
    if (tdim==.)       tdim = 10
    if (pr_good==.)    pr_good = .5
    if (newscycle==.)  newscycle = 20
    if (Nagents==.)    Nagents = 1000
    if (baseclicks==.) baseclicks = 2*Nagents/newscycle
    if (botclicks==.)  botclicks = 0
    
    agents.reinit("real", 2)
    items.reinit("real", 3)
    pr.reinit("real", 1)
}

void clicks::populate_news() 
{
    real scalar i
    real rowvector key
    
    for(i=1; i <= newscycle ; i++) {
        key = i, 0, `content'
        items.put(key,runiform(1,1)<pr_good)
        key = i, 0, `clicks'
        items.put(key,i*ceil(runiform(1,1)*baseclicks))
    }
}

void clicks::populate_agents()
{
    real scalar i
    real rowvector key
    
    for(i=1; i<=Nagents ; i++) {
        key = i,0
        agents.put(key,ceil(runiform(1,1)*10))
    }

}

void clicks::run()
{
    real scalar t , i
    
    setup()
    populate_news()
    populate_agents()
    
    for(t=1; t <= tdim; t++) {
        news_step(t)
        pr(t)
        for(i=1; i <= Nagents; i++) {
            agent_step(i,t)
        }
    }
}

end


mata
    foo = clicks()
    foo.tdim(200)
    foo.pr_good(.5)
    foo.botclicks(50)
    foo.run()
    foo.mood(("m1","m2","m3","m4","m5","m6","m7","m8","m9","m10"))
end
gen t = _n-1
sort t 
forvalues i = 2/10 {
    local j = `i' - 1
    replace m`i' = m`i' + m`j'
}

twoway area m10 t, color("165   0  38") || ///
       area m9  t, color("215  48  39") || ///
       area m8  t, color("244 109  67") || ///
       area m7  t, color("253 174  97") || ///
       area m6  t, color("254 224 144") || ///
       area m5  t, color("224 243 248") || ///
       area m4  t, color("171 217 233") || ///
       area m3  t, color("116 173 209") || ///
       area m2  t, color(" 69 117 180") || ///
       area m1  t, color(" 49  54 149")    ///
       plotregion(margin(zero))            /// 
       name(mood, replace)                 /// 
       legend(order(1  "10 good"           ///
                    2   "9"                ///
                    3   "8"                ///
                    4   "7"                ///
                    5   "6"                ///
                    6   "5"                ///
                    7   "4"                ///
                    8   "3"                ///
                    9   "2"                ///
                    10  "1 bad" ))
drop _all
mata foo.items(("id", "t", "what", "val"))
reshape wide val, i(id t) j(what)

gen clicks_good = val2*val1
gen clicks_bad  = val2*(1-val1)
collapse (sum) clicks_good clicks_bad, by(t)
twoway line clicks_good clicks_bad t,   ///
    lcolor("165   0  38" " 49  54 149") ///
    lpattern(solid solid)               ///
    legend(order(1 "good" 2 "bad"))     ///
    ytitle(total number of clicks)
