clear all

do abm_grid.mata

mata
mata set matastrict on

class grid extends abm_grid
{

}

class schelling {
    private:
        // all variables are private
        class grid             scalar universe
        class AssociativeArray scalar agents

        real   scalar    rdim
        real   scalar    cdim
        real   scalar    tdim
        real   scalar    limit
        real   scalar    nblue
        real   scalar    nred
        real   scalar    nagents
        
        void             is_posint()
        
        // initialize the model
        void             setup_agents()
        void             setup_universe()        
        void             populate_universe()
        
        string scalar    get_color()
        real   scalar    sum_neighbours()    // proportion of same color in neighbourhood
        real   scalar    is_happy()         // is that proportion larger than limit 
        void             move()                // search for nearest empty spot where agent would be happy
        void             step()                // a single step by a single agent
        

    
    public:
        // set or return settings
        void             rdim()
        void             cdim()
        void             tdim()
        void             limit()
        void             nblue()
        void             nred()
        
        // run the model
        void             run()
        
        // some functions to export or summarize the results
}

// -------------------------------------------------------------- error checking
void schelling::is_posint(real scalar val, string scalar name) 
{
    string scalar errmsg
    
    if (val <= 0 | mod(val,1)!= 0 ){
        errmsg = name + " must be a positive integer"
        _error(3300, errmsg)
    }
}

// ------------------------------------------------------------------ setup grid
void schelling::rdim(real scalar dim) 
{
    is_posint(dim, "dim")    
    universe.rdim(dim)
    rdim = dim
}
end
