clear all

do abm_grid.mata

mata
mata set matastrict on

class grid extends abm_grid
{

}

class schelling {
    private:
        // all variables are private
        class grid             scalar universe
        class AssociativeArray scalar agents

        real   scalar    rdim
        real   scalar    cdim
        real   scalar    tdim
        real   scalar    limit
        real   scalar    nblue
        real   scalar    nred
        real   scalar    nagents
        
        void             is_posint()
        void             is_prob()
        
        // initialize the model
        void             setup_agents()
        void             setup_universe()        
        void             populate_universe()
        
        string scalar    get_color()
        real   scalar    sum_neighbours()    // proportion of same color in neighbourhood
        real   scalar    is_happy()         // is that proportion larger than limit 
        void             move()                // search for nearest empty spot where agent would be happy
        void             step()                // a single step by a single agent
        

    
    public:
        // set or return settings
        transmorphic     rdim()
        transmorphic     cdim()
        transmorphic     tdim()
        transmorphic     limit()
        transmorphic     nblue()
        transmorphic     nred()
        
        // run the model
        void             run()
        
        // some functions to export or summarize the results
}

// -------------------------------------------------------------- error checking
void schelling::is_posint(real scalar val, string scalar name) 
{
    string scalar errmsg
    
    if (val <= 0 | mod(val,1)!= 0 ){
        errmsg = name + " must be a positive integer"
        _error(3300, errmsg)
    }
}
void schelling::is_prop(real scalar val, string scalar name) 
{
    string scalar errmsg
    
    if (val < 0 | val > 1 ){
        errmsg = name + " must be between or including 0 and 1"
        _error(3300, errmsg)
    }
}

// ------------------------------------------------------------------ setup grid
transmorphic schelling::rdim(| real scalar dim) 
{
    if (args() == 1) {
        is_posint(dim,"dim")
        universe.rdim(dim)
        rdim = dim
    }
    else {
        return(universe.rdim())
    }
}
transmorphic schelling::cdim(| real scalar dim) 
{
    if (args() == 1) {
        is_posint(dim,"dim")
        universe.cdim(dim)
        cdim = dim
    }
    else {
        return(universe.cdim())
    }
}
transmorphic schelling::tdim(| real scalar dim) 
{
    if (args() == 1) {
        is_posint(dim,"dim")
        universe.tdim(dim)
        tdim = dim
    }
    else {
        return(universe.tdim())
    }
}
void schelling::setup_universe()
{
    if (tdim()==.) {
        _error(3000, "tdim must be set first")
    }
    universe.setup()
}    

// ---------------------------------------------------------------- setup agents
transmorphic schelling::limit(| real scalar val) 
{
    if (args() == 1) {
        is_prop(val,"value")
        limit = val
    }
    else {
        return(limit)
    }
}
transmorphic schelling::nblue(| real scalar val) 
{
    if (args() == 1) {
        is_posint(val,"value")
        nblue = val
    }
    else {
        return(nblue)
    }
}
transmorphic schelling::nred(| real scalar val) 
{
    if (args() == 1) {
        is_posint(val,"value")
        nred = val
    }
    else {
        return(nred)
    }
}
end
