//layout toc link(subsection) secbold 
//layout bottombar toc

//titlepage --------------------------------------------------------------------
//title Agent Based Models in Mata

/*txt

{center:Maarten Buis}
{center:University of Konstanz}

{center:maarten.buis@uni.kn}
txt*/
//endtitlepage

/*toctitle
Table of Contents
toctitle*/

//section Introduction
//subsection What is an Agent Based Model?
//slide ------------------------------------------------------------------------
//title What is an Agent Based Model?

/*txt
{pstd}
An Agent Based Model is a simulation in which agents, that each follow simple rules, 
interact with one another and thus produce a often surprising outcome at the macro 
level. 

{pstd}
The purpose of an ABM is to explore mechanisms through which actions of the 
individual agents add up to a macro outcome, by varying the rules that agents have
to follow or varying with whom the agent can interact. 
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Example 1: Immunization rate

/*txt
{pstd}
Below is an example Agent Based Model that illustrates how a sufficiently high
immunization rate protects even the non-immunized individuals in a population, 
and also allows one to explore at what immunization rate this protection 
disappears.

{pstd}
In this model a single agent lives in each cell of a square grid and does not 
move. An agent could have 5 states. She could be:

{pmore}immunized{p_end}
{pmore}not-immunized and vulnerable{p_end}
{pmore}not-immunized and infected{p_end}
{pmore}not-immunized and infectious{p_end}
{pmore}not-immunized, healed, and no longer vulnerable{p_end} 

{pstd}
In the first round a single random vulnerable agent becomes infected. 

{pstd} 
In all subsequent rounds 

{pmore}all infected agents become infectious, and {p_end}
{pmore}infect all vulnerable agents within {it:radius} cell radius.{p_end}
{pmore}All infectious agents become healed and no longer vulnerable.{p_end}

{pstd}
{stata "db immune" :run this model}
txt*/
//file immune.dlg
//file infection.mata    
//file abm_grid.mata


//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Example 2: Schellings segregation model

/*txt
{pstd}
In this model there are blue and red agents, that live in cells on a square grid.
Each cell can contain only one agent, but not all cells are occupied. An agent
is happy if the proportion of agents of the same colour in her neighbourhood is
above a certain cut-off value. If she is unhappy she will move to the nearest
available cell that does satisfy her needs. 

{pstd}
The point of this simulation that even with a low cut-off value, like 30%, you 
will get fairly seggregated neighbourhoods.

{pstd}
{stata "db schelling" :run this model}
txt*/
//file schelling.mata
//file schelling.dlg

//endslide ---------------------------------------------------------------------


//slide ------------------------------------------------------------------------
//title Example 3: Bots influencing the mood

/*txt
{pstd}
In this simulation there are agents that have a mood ranging from 10 (good) to
1 (bad). 

{pstd}
In each iteration an agent reads a single newsitems. If the newsitem is 
good it increases the mood, and if it is bad it decreases the mood. 

{pstd}
The chance of choosing an item is proportional to the number of agents that have 
previously read that article. 

{pstd}
An item disappears after a certain number of iterations 
and a new newsitem is created with a random number of initial "clicks".

{pstd}
A "bot" tries to decrease the mood by adding additional clicks to the initial 
clicks of bad newsitems.

{pstd}
{stata "db trolls" :run this model}
txt*/
//file abm_clicks.mata
//file trolls.dlg

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title What is this talk about?

/*txt
{pstd}
This talk reports on an experiment to find out if it is viable to do ABMs in 
Stata.

{pstd}
In particular many ABMs have tasks in common, e.g. in many cases agents "live" 
inside a square grid and we want to know who their "neighbours" are. How can 
we create and maintain a library of these common functions. This library needs 
to be:

{pmore}
convenient to "import" in your model

{pmore}
modular

{pmore}
easy to maintain and extend

txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title The alternatives

/*txt
{pstd}
There are various dedicated environments for creating ABMs. 

{pmore}
{bf:netlogo} (netlogo) {browse "http://ccl.northwestern.edu/netlogo/"}

{pmore}
{bf:repast} (java, python, C++, and others) {browse "https://repast.github.io/"}

{pmore}
{bf:mason} (java) {browse "https://cs.gmu.edu/~eclab/projects/mason/"}

{pmore}
{bf:mesa} (python) {browse "https://mesa.readthedocs.io/"}

{pstd}
People who primarily create ABMs are not going to move away from those, and 
rightly so.

{pstd}
So the toolkits discussed in this presentation should be aimed at people who 
primarily use Stata and occationally want to make an ABM.

{pstd}
The question becomes: can we make making ABMs in Stata easier/quicker/more 
convenient for this target audience than learning one of these environments.
txt*/

//endslide ---------------------------------------------------------------------

//subsection Mata
//slide ------------------------------------------------------------------------
//title Entering and exiting Mata, scalars, matrices

/*txt
{pstd}
For implementing ABMs we will be using Stata's matrix language Mata

{pstd}
Here I give only a very brief introduction, for much more see:

{pmore}
William W. Gould  (2018) {it:The Mata Book: A Book for Serious Programmers and Those Who Want to Be},
College Station, TX: Stata Press.


{pstd}
You enter Mata by typing {cmd:mata} in Stata, and you leave it by typing {cmd:end}

{pstd}
In it you can work with matrices
txt*/

//ex
mata
foo = 3
foo

foo = 1, 2 \ 3, 4
foo
 
bar = J(2,2,1)
bar
foo + bar
end
//endex

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Associative Array

/*txt
{pstd}
However, matrices aren't the only thing that Mata can work with.

{pstd}
An ABM consist of agents that have properties, maybe a grid on which agents 
live, etc.

{pstd}
A lot of the work in ABMs is just storing those properties, reading them, and 
changing them. 

{pstd}
We could do that in matrices, but {it:associative arrays} tend to be more flexible

{pstd}
In an associative array you store key and value pairs. In the example below the 
key is the last name, while the value is the first name. The value could be 
anything.
txt*/

//ex
mata
name = AssociativeArray()
name.put("Cox", "Nicholas")
name.put("Jann", "Ben")
name.put("Buis", "Maarten")

name.get("Jann")
end
//endex

/*txt
{pstd}
The key can also be a vector, leading to a 2, 3, or higher dimensional array.
txt*/

//ex
mata
name.reinit("string",2)
name.put(("Cox","middle"),"J.")
name.put(("Cox", "first"), "Nicholas")
name.put(("Jann", "first"), "Ben")
name.put(("Buis", "middle"), "Leendert")
name.put(("Buis", "first"), "Maarten")

name.get(("Cox", "middle"))
end
//endex


//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Writing functions

/*txt
{pstd}
When we write a new function we need to tell Mata what "kind of thing" it can
expect that function to return, and the kind of things the function can expect 
as inputs

{pstd}
Things (scalars, matrices) we create inside a function are local to that function

{pstd}
It is good practice to also first declare what kind of thing it is before 
creating it.
txt*/

//ex
clear mata

mata:

real scalar sum2(real scalar a, real scalar b)
{
    real scalar result
    
    result = a + b
    
    return(result)
}

sum2(2,4)
end
//endex


//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Loops, if

/*txt
{pstd}
Looping and conditional execution are basic actions happing sometime in any 
program

{pstd}
To illustrate the syntax used in Mata I add all even numbers between 1 and 10

txt*/
//ex
mata
result = 0
for(i=1 ; i<=10; i++) {
    if (mod(i,2)==0) {
        result = result + i
    }
}
result
end
//endex

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title structure

/*txt
{pstd}
If you want to implement a larger project, like an ABM, then you will want to 
split it up in sub-routines rather than put it all in one big program. 

{pstd}
However, you will often have quite a lot of "raw material" that you want to share
between sub-routines

{pstd}
In that case it is convenient to use a {cmd:structure} to organize that raw 
material on a "tray", and pass that tray to the sub-routines.
txt*/

//ex
clear mata

mata:
// ------------- define what goes where on our "tray"
struct datastruct {
    real scalar a, b
}

// ------------------------------------ main function
void arithmatic(real scalar a, real scalar b)
{
    // create the tray called data 
    struct datastruct scalar data
    
    // fill that tray
    data.a = a
    data.b = b
    
    // pass that tray on to sub-routines
    sum2(data)
    prod2(data)
}

// --------------------------------- the sub-routines
real scalar sum2(struct datastruct scalar data)
{
    return(data.a + data.b)
}

real scalar prod2(struct datastruct scalar data)
{
    return(data.a * data.b)
}

// ---------------------------- use the main function
arithmatic(2,4)
end
//endex

//endslide ---------------------------------------------------------------------


//slide ------------------------------------------------------------------------
//title Class

/*txt
{pstd}
The next step is to use a class, which you can think of as a structure that can 
also contain functions

{pstd}
The functions are local to the class

{pstd}
Each function has access to all material stored in the class

{pstd}
An "outsider", the user, can only interact with the public functions, protected
and private functions and variables can only be accessed from within the class.
txt*/

//ex
clear mata

mata:

class arithmatic {
    protected:
        real scalar a
        real scalar b
        real scalar sum()
        real scalar prod()
    
    public:
        void input()
        void run()
}

void arithmatic::input(real scalar aval, real scalar bval)
{
    a = aval
    b = bval
}

real scalar arithmatic::sum()
{
    return(a + b)
}

real scalar arithmatic::prod()
{
    return(a * b)
}

void arithmatic::run()
{
    sum()
    prod()
}

// how the user interacts with this program
math = arithmatic()
math.input(2,4)
math.run()
end
//endex

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Inheritance

/*txt
{pstd}
Classes can inherrit from other classes: All variables and functions in the 
parent class also exist in the child class

{pstd}
The child class can add new functions

{pstd}
Function in the child class with the same name hide the function from the parent.

{pstd}
That is a model for distributing common functions in ABMs
txt*/

//ex
mata
class arith extends arithmatic 
{
    private:
        real scalar ratio()
    
    public:
        void run()
}

real scalar arith::ratio()
{
    return(a/b)
}

void arith::run() {
    sum()
    prod()
    ratio()
}

math = arith()
math.input(2, 4)
math.run()
end
//endex

//endslide ---------------------------------------------------------------------


//subsection The abm_grid class
//slide ------------------------------------------------------------------------
//title Setting up a grid and finding neighbours

/*txt
{pstd}
This class helps create and use a grid

{pstd}
The {cmd:abm_grid} class is available from GitHub: 
{browse "https://github.com/maartenteaches/abm_grid"}

{pstd}
The main file is {cmd:abm_grid.mata}, all other files there are help files.

{pstd}
{help abm_grid}

{pstd}
Below is an example that uses {cmd:abm_grid} to create a 10 by 10 grid that is 
constant over time and allows for 1 agent per cell
txt*/
//file abm_grid.sthlp
//file abm_grid_agent_id.sthlp
//file abm_grid_agent_ids.sthlp
//file abm_grid_agent_loc.sthlp
//file abm_grid_basecoords.sthlp
//file abm_grid_baserings.sthlp
//file abm_grid_copy_agent.sthlp
//file abm_grid_copy_agents.sthlp
//file abm_grid_copy_grid.sthlp
//file abm_grid_create_agent.sthlp
//file abm_grid_delete_agent.sthlp
//file abm_grid_delete_agents.sthlp
//file abm_grid_error.sthlp
//file abm_grid_extract.sthlp
//file abm_grid_findring.sthlp
//file abm_grid_has_agent.sthlp
//file abm_grid_make_key.sthlp
//file abm_grid_move_agent.sthlp
//file abm_grid_neumann.sthlp
//file abm_grid_protected.sthlp
//file abm_grid_randit.sthlp
//file abm_grid_random_cell.sthlp
//file abm_grid_rdim.sthlp
//file abm_grid_schedule.sthlp
//file abm_grid_setup.sthlp
//file abm_grid_size.sthlp
//file abm_grid_torus.sthlp

//ex
clear mata
run abm_grid.mata

mata 
grid = abm_grid()
grid.rdim(10)
grid.cdim(10)
grid.setup()
end
//endex

/*txt
{pstd}
We use a grid in ABMs because it defines who the neighbours of an agent are.
txt*/

//ex
mata
grid.find_ring(3,3,1)
grid.find_ring(3,1,1)
end
//endex

/*txt
{cmd}
    | 1  2  3  4      | 1  2  3  4  
  --+------------   --+------------ 
  1 |               1 |             
  2 |    5  4  3    2 | 4  3  
  3 |    6  o  2    3 | o  2   
  4 |    7  8  1    4 | 5  1  
{txt}

{pstd}
By default the cells that are outside the grid are ignored, but that can be 
changed to let the grid "wrap around".
txt*/

//ex
mata
grid.torus(1)
grid.setup()

grid.find_ring(3,1,1)
end
//endex

/*txt
{cmd}
    | 1  2  3  4 .. 10 
  --+----------------- 
  1 |             
  2 | 4  3           5   
  3 | o  2           6   
  4 | 8  1           7  
{txt}

{pstd}
By default a neighbourhood with radius 1 is defined as all cells 1 step removed,
where a step could be horizontal, vertical, {it:or} diagonal. We could change 
that to allow only horizontal and vertical steps.
txt*/

//ex
mata
grid.neumann(1)
grid.setup()

grid.find_ring(3,3,1)
end
//endex

/*txt
{cmd}
    | 1  2  3  4 
  --+------------
  1 |            
  2 |       3   
  3 |    4  o  2 
  4 |       1   
{txt}
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Managing agents

/*txt
{pstd}
In the grid we only store the agent id, the properties of those agents (e.g. her
color for Schelling seggregation model) should be stored in a seperate 
associative array

{pstd}
The {cmd:abm_grid} class has function to check if an agent exists on a give cell,
to create an agent on a cell, and to retrieve the agent id of the agent living 
at a cell.
txt*/

//ex
mata
grid.has_agent(1,1)
grid.create_agent(1,1,5)
grid.has_agent(1,1)
grid.agent_id(1,1)
end
//endex

/*txt
{pstd}
The {cmd:abm_grid} class also has functions to move, copy, or delete agents 
txt*/

//ex
mata
//move agents
grid.move_agent(1,1,2,2)
grid.has_agent(1,1)
grid.has_agent(2,2)
grid.agent_id(2,2)

//copy agents
grid.copy_agent(2,2,1,1)
grid.agent_id(2,2)
grid.agent_id(1,1)

// delete agents
grid.delete_agent(2,2)
grid.has_agent(2,2)
end
//endex

/*txt
{pstd}
After the ABM has run we often want to export the results to Stata to create 
graphs


txt*/

//ex
mata
grid.create_agent(5,3,2)
grid.create_agent(2,9,1)
grid.extract()
end
//endex
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Adding a time dimension to the grid

/*txt
{pstd}
We need to add a time dimension to our grid if agents can move around on the 
grid, e.g. in Schelling's segregation model.
txt*/

//ex
mata
grid.tdim(5)
grid.setup()

// we setup the intial grid
grid.create_agent(1,1,5, 0)
grid.create_agent(3,4,1, 0)
grid.create_agent(4,4,2, 0)

// next iteration we first copy the grid
grid.copy_grid(0,1)
// and make changes as necessary
grid.move_agent(1,1,2,3,1)

//at the end we can recover the entire history of the grid:
grid.extract()
end
//endex

/*txt
{pstd}
If the agents cannot move, e.g. in the immunization model, then grid does not need
a time dimension.

{pstd}
Now the associative array storing the agents needs to have a time dimension.
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Multiple agents per cell

/*txt
{pstd}
For some ABMs we want multiple agents to live in the same cell

{pstd}
For example, a predator-prey model where sheep and wolve agents wander on a grid

{pstd}
Each agent still needs a distinct "adress", so we can add a "location" dimension
within cells 
txt*/

//ex
mata
grid.idim(3)
grid.setup()

// add three agents to the cell 1,1 at itereation 0
grid.create_agent(1,1,5, 0,1)
grid.create_agent(1,1,6, 0,2)
grid.create_agent(1,1,7, 0,3)

grid.agent_id(1,1,0,1)
grid.agent_ids(1,1,0)
end
//endex
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title How do I use it?

/*txt
{pstd}
A typical start of an ABM would be:
txt*/

//ex
clear mata
run abm_grid.mata

mata
class grid extends abm_grid
{
    
}

class abm
{
    private:
        class grid scalar universe
        
    // other things required for the model
}
end    
//endex
//endslide ---------------------------------------------------------------------

//section Building Agent Based Models
//subsection the Schellings segregation model
//slide ------------------------------------------------------------------------
//title How to start

/*txt
{pstd}
I start with a rough idea of what the model should be, e.g.: 

{pmore}
In this model there are blue and red agents, that live in cells on a square grid.
Each cell can contain only one agent, but not all cells are occupied. An agent
is happy if the proportion of agents of the same colour in her neighbourhood is
above a certain cut-off value. If she is unhappy she will move to the nearest
available cell that does satisfy her needs. 

{pstd}
From that I start defining the class, what kind of variables do I need to store,
what kind of functions do I need.

{pstd}
This class definition becomes the todo list for the project.

//codefile schelling1.mata "doedit schelling1.mata"
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Receiving and returning settings

/*txt
{pstd}
We want the user to be able to specify 

{pmore}the number of rows and columns in the grid{p_end}
{pmore}the number of iterations{p_end}
{pmore}the number of red and blue agents{p_end}
{pmore}the limit, below which the agent becomes unhappy{p_end}

{pstd}
Lets start with the number of rows: 

//codefile schelling2.mata "doedit schelling2.mata"

{pstd}
We are checking the input to see if it is a positive integer, and we will have
to do that for most of the other settings as well. Lets put that in a separate 
function: 

//codefile schelling3.mata "doedit schelling3.mata"

{pstd}
Can't we make it such that if we give {cmd:rdim()} an argument it sets {it:rdim}
to that value, but we don't give {cmd:rdim()} an argument it returns the value 
of {it:rdim}? 

//codefile schelling4.mata "doedit schelling4.mata"

{pstd}
That looks good, just extend to all other settings 

//codefile schelling5.mata "doedit schelling5.mata"
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Setting up
/*txt
{pstd}
Seting up the universe is something we can let {cmd:abm_grid} do, the only thing
we need to add is to make sure {it:tdim} is specified: 

//codefile schelling6.mata "doedit schelling6.mata"

{pstd}
To setup the agents we first need to check some of the options: 

//codefile schelling7.mata "doedit schelling7.mata"

{pstd}
Now we need to change the {it:agents} associative array such that the key is the
agent id (a real number) and fill it with red and blue agents:


//codefile schelling8.mata "doedit schelling8.mata"

{pstd}
The final step is put these agents on random places on the grid in the initial 
iteration: 

//codefile schelling9a.mata "doedit schelling9a.mata"
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Summarize the neighbourhood

/*txt
{pstd}
Now we agents living on the grid. The next step would be for them to look around
and see if they are happy with their neighbourhood.

{pstd}
Lets start with a routine that returns the color of an agent living at a certain
cell: 

//codefile schelling9b.mata "doedit schelling9b.mata"

{pstd}
Next we loop over all neighbours and count the proportion of same color neighbours:

//codefile schelling10.mata "doedit schelling10.mata"

{pstd}
Right now a missing value will be returned if an agent has no neighbours, we 
need to fix that : 

//codefile schelling11.mata "doedit schelling11.mata"

{pstd}
We are later also going to use this function to find new locations. In that case
the focal cell is not the cell where the agent currently lives: 

//codefile schelling12.mata "doedit schelling12.mata"

{pstd}
In that case the neighbourhood of the focal cell could contain the cell where the
agent currently lives. We would want to exclude that cell, as she will be moving:

//codefile schelling13.mata "doedit schelling13.mata"

{pstd}
Now we can create another function that runs {cmd:sum_neighbourhood()} and 
compares the returned proportion with the limit: 

//codefile schelling14.mata "doedit schelling14.mata"
txt*/


//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Move to a better cell

/*txt
{pstd}
There is an function in {cmd:abm_grid} that can move an agent from one location 
to another, so we only need to focus on finding the nearest location that will
make the agent happy.

{pstd}
Lets start with a loop that moves over all cells in the grid in an outward spiral:

//codefile schelling15.mata "doedit schelling15.mata"

{pstd}
Now we can check any empty cells to see if the agent will be happy there, and 
move when that is the case: 

//codefile schelling16.mata "doedit schelling16.mata"

{pstd}
We can stop the loop the first time we find a appropriate cell:

//codefile schelling17.mata "doedit schelling17.mata"

txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Run the simulation

/*txt 
{pstd}
First we will define the action of a single agent at each
itteration: 

//codefile schelling18.mata "doedit schelling18.mata"

{pstd}
Next we will define the function {cmd:run()} that lets the user start the 
simulation. It starts by setting up the universe and agents, populate that 
universe: 

//codefile schelling19.mata "doedit schelling19.mata"

{pstd}
Than it loops over iterations, each time copying the grid to the next iteration, 
and than looping over the cells of that grid: 

//codefile schelling20.mata "doedit schelling20.mata"

{pstd}
If there is an agent in that cell it lets it take its step: 

//codefile schelling21.mata "doedit schelling21.mata"
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title display summary statistics

/*txt
{pstd}
We want to display a table that shows for each iteration the average proportion 
of same color neighbours and the proportion of happy agents.

{pstd}
Start with creating a matrix with the first column indicating the iteration: 

//codefile schelling22.mata "doedit schelling22.mata"

{pstd}
Next we fill in the second column with the average proportion of same color 
agents: 

//codefile schelling23.mata "doedit schelling23.mata"

{pstd}
Now we can fill the last column: 

//codefile schelling24.mata "doedit schelling24.mata"

{pstd}
If export this matrix to Stata then we can use {cmd:matlist} to make this table 
look nice: 

//codefile schelling25.mata "doedit schelling25.mata"
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Export to Stata
/*txt
{pstd}
To plot the agents on the grid we first need to export it to Stata. We would 
like for each cell with an agent the: 

{pmore}row number{p_end}
{pmore}column number{p_end}
{pmore}iteration number{p_end}
{pmore}id of that agent{p_end}
{pmore}the color of that agent{p_end}
{pmore}whether or not the agent is happy{p_end}

{pstd}
So this function must receive 6 names of variables that are to be created

{pstd}
With the {cmd:abm_grid} class we can extract the row, column, iteration number 
and the id, though the 4{sup:th} column is superfluous: That contains the 
location within the cell, but we only allowed one agent per cell 

//codefile schelling26.mata "doedit schelling26.mata"

{pstd}
The 5{sup:th} column is the id of the agent, which we can use to recover the 
color from the associative array {it:agents}: 

//codefile schelling27.mata "doedit schelling27.mata"

{pstd}
We can now add the whether or not the agent is happy: 

//codefile schelling28.mata "doedit schelling28.mata"

{pstd}
Now we need to store that matrix as a Stata dataset: 

//codefile schelling29.mata "doedit schelling29.mata"
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Plot the grid
/*txt
{pstd}
We can use {cmd twoway scatter} to plot the grid for the initial iteration: 

//codefile schelling.mata "doedit schelling30.mata"

{pstd}
There are still some improvements necessary:

{pmore}We don't need to label the axes{p_end}
{pmore}It is a square grid, so the graph should also be square{p_end}
{pmore}We could add lines to highlight the cells{p_end}

//codefile schelling31.mata "doedit schelling31.mata"

{pstd}
We can improve the graph further:

{pmore}We can distinguish between happy and non-happy agents{p_end}
{pmore}We can add the agent id{p_end}

//codefile schelling32.mata "doedit schelling32.mata"

{pstd}
Now we can use a loop to create the same graph for all iterations: 

//codefile schelling33.mata "doedit schelling33.mata"
txt*/
//endslide ---------------------------------------------------------------------

//subsection Other models
//slide ------------------------------------------------------------------------
//title Other models
/*txt
{pstd}
The code for the immunization model is here: 

//codefile infection2.mata "doedit infection2.mata"

{pstd}
The code for the bot and public mood model is here:

//codefile abm_clicks2.mata "doedit abm_clicks2.mata"
 txt*/
//endslide ---------------------------------------------------------------------

//section Conclusion
//slide ------------------------------------------------------------------------
//title Conclusion

/*txt
{pstd}
Implementing and agent based model in Mata is doable

{pstd}
It is a non-trivial programming exercise, but that is to some extend true for 
all platforms

{pstd}
Creating a GUI tends to be easier in the other platforms, though that does not 
worry me that much. Once the model has been programmed, the interaction with that 
model is intuitive enough.

{pstd}
Creating simple tables and graphs also tends to be easier in the other platforms. 
I find that more serious.
txt*/
//endslide ---------------------------------------------------------------------
